/**
 * abstract syntax tree for expressions
 */
package expression;

/** the abstract syntax tree representation
 */
public abstract class Tree {

    /** the source code position corresponding to this tree node
     */
    int pos;
    
    public Tree(int pos) {
        this.pos = pos;
    }
    
    /** the method for applying visitors to trees
     */
    public abstract void apply(Visitor v);

    /** the different abstract syntax tree nodes
     */
    public static class Program extends Tree {
        Tree[] defs;

        public Program(int pos, Tree[] defs) {
            super(pos);
	    this.defs = defs;
        }
        
        public void apply(Visitor v) {
            v.caseProgram(this);
        }
    }

    public static class Def extends Tree {
	String name;
	Tree expr;

	Symbol sym;

        public Def(int pos, String name, Tree expr) {
            super(pos);
	    this.name = name;
	    this.expr = expr;
        }
        
        public void apply(Visitor v) {
            v.caseDef(this);
        }
    }

    public static class IntLiteral extends Tree {
	int i;

        public IntLiteral(int pos, int i) {
            super(pos);
	    this.i = i;
        }
        
        public void apply(Visitor v) {
            v.caseIntLiteral(this);
        }
    }

    public static class Ident extends Tree {
	String name;

	Symbol sym;

        public Ident(int pos, String name) {
            super(pos);
	    this.name = name;
        }
        
        public void apply(Visitor v) {
            v.caseIdent(this);
        }
    }

    public static class BinOp extends Tree {
	char op;
	Tree left;
	Tree right;

        public BinOp(int pos, char op, Tree left, Tree right) {
            super(pos);
	    this.op = op;
	    this.left = left;
	    this.right = right;
        }
        
        public void apply(Visitor v) {
            v.caseBinOp(this);
        }
    }

    public static class Bad extends Tree {
        
        public Bad(int pos) {
            super(pos);
        }
        
        public void apply(Visitor v) {
            v.caseBad(this);
        }
    }
    
    /** the visitor interface for trees
     */
    public interface Visitor {
	void caseProgram(Program tree);
	void caseDef(Def tree);
	void caseIdent(Ident tree);
        void caseIntLiteral(IntLiteral tree);
        void caseBinOp(BinOp tree);
        void caseBad(Bad tree);
    }
}

